@@ -4,7 +4,7 @@ use warnings;
 use 5.006;
 
 package HTTP::BrowserDetect;
-$HTTP::BrowserDetect::VERSION = '1.76';
+$HTTP::BrowserDetect::VERSION = '1.77';
 use vars qw(@ALL_TESTS);
 
 # Operating Systems
@@ -71,7 +71,7 @@ our @BROWSER_TESTS = qw(
     konqueror     realplayer  netfront
     mobile_safari obigo       aol
     lotusnotes    staroffice  icab
-    webtv
+    webtv         browsex     silk
 );
 
 our @IE_TESTS = qw(
@@ -184,6 +184,7 @@ my %ROBOT_NAMES = (
 my %BROWSER_NAMES = (
     aol           => 'AOL Browser',
     blackberry    => 'BlackBerry',
+    browsex       => 'BrowseX',
     chrome        => 'Chrome',
     curl          => 'curl',
     dsi           => 'Nintendo DSi',
@@ -192,6 +193,7 @@ my %BROWSER_NAMES = (
     icab          => 'iCab',
     iceweasel     => 'IceWeasel',
     ie            => 'MSIE',
+    konqueror     => 'Konqueror',
     links         => 'Links',
     lotusnotes    => 'Lotus Notes',
     lynx          => 'Lynx',
@@ -205,6 +207,7 @@ my %BROWSER_NAMES = (
     puf           => 'puf',
     realplayer    => 'RealPlayer',
     safari        => 'Safari',
+    silk          => 'Silk',
     staroffice    => 'StarOffice',
     webtv         => 'WebTV',
 );
@@ -486,6 +489,13 @@ sub _init_core {
             $browser_tests->{AOL} = 1;
         }
     }
+    elsif ( index( $ua, "silk" ) != -1 ) {
+
+        # Has to go above Chrome, it includes "like Chrome/"
+
+        $browser = 'SILK';
+        $browser_tests->{$browser} = 1;
+    }
     elsif ( index( $ua, "chrome/" ) != -1 ) {
 
         # Browser is Chrome
@@ -602,10 +612,14 @@ sub _init_core {
     elsif ( index( $ua, "nintendo dsi" ) != -1 ) {
         $browser = 'DSI';     # Test gets set during device check
     }
-    elsif ( index( $ua, "obigo/" ) != -1 ) {
+    elsif ( index( $ua, "obigo" ) != -1 ) {
         $browser = 'OBIGO';
         $browser_tests->{$browser} = 1;
     }
+    elsif ( index( $ua, "browsex" ) != -1 ) {
+        $browser = 'BROWSEX';
+        $browser_tests->{$browser} = 1;
+    }
     elsif ( index( $ua, "libcurl" ) != -1 ) {
         $browser = 'CURL';    # Test gets set during robot check
     }
@@ -618,7 +632,7 @@ sub _init_core {
     # Other random tests
 
     $tests->{JAVA} = 1
-        if ( index( $ua, "java" ) != -1
+        if ( $ua =~ m{\bjava}
         || index( $ua, "jdk" ) != -1
         || index( $ua, "jakarta commons-httpclient" ) != -1 );
     $tests->{X11}    = 1 if index( $ua, "x11" ) != -1;
@@ -644,7 +658,7 @@ sub _init_core {
             $self->{realplayer_version} = $1;
             ( $self->{major}, $self->{minor} )
                 = split( /\./, $self->{realplayer_version} );
-            $self->{minor} = ".$self->{minor}" if $self->{minor};
+            $self->{minor} = ".$self->{minor}" if defined( $self->{minor} );
         }
         elsif ( $ua =~ /realplayer\s(\w+)/ ) {
             $self->{realplayer_version} = $1;
@@ -779,14 +793,20 @@ sub _init_robots {
         ||= $r
         || $tests->{JAVA}
         || index( $ua, "agent" ) != -1
+        || index( $ua, "appender" ) != -1
         || index( $ua, "bot" ) != -1
+        || index( $ua, "checker" ) != -1
         || index( $ua, "copy" ) != -1
         || index( $ua, "crawl" ) != -1
+        || index( $ua, "explorador" ) != -1
         || index( $ua, "fetch" ) != -1
         || index( $ua, "find" ) != -1
         || index( $ua, "ia_archive" ) != -1
         || index( $ua, "index" ) != -1
+        || index( $ua, "sleuth" ) != -1
         || index( $ua, "reap" ) != -1
+        || index( $ua, "scan" ) != -1
+        || index( $ua, "service" ) != -1
         || index( $ua, "spider" ) != -1
         || index( $ua, "worm" ) != -1
         || index( $ua, "zyborg" ) != -1
@@ -1208,6 +1228,27 @@ sub _init_version {
             $minor = $2;
         }
     }
+    elsif ( $browser eq 'BROWSEX' ) {
+        if ( $ua =~ m{BrowseX \((\d+)\.(\d+)([\d.]*)}i ) {
+            $major = $1;
+            $minor = $2;
+            $beta  = $3;
+        }
+    }
+    elsif ( $browser eq 'SILK' ) {
+        if ( $ua =~ m{Silk/(\d+)\.(\d+)([\d.]*)}i ) {
+            $major = $1;
+            $minor = $2;
+            $beta  = $3;
+        }
+    }
+    elsif ( $browser eq 'OBIGO' ) {
+
+        # We have no working obigo version tests, so give up as opposed
+        # to setting wrong information.
+        $major = "0";
+        $minor = ".0";
+    }
 
     if ( !defined($major) ) {
 
@@ -1458,6 +1499,7 @@ sub _init_device {
             || index( $ua, "symbos" ) != -1
             || index( $ua, "opera mobi" ) != -1
             || index( $ua, "fennec" ) != -1
+            || index( $ua, "obigo" ) != -1
             || index( $ua, "opera tablet" ) != -1
             || index( $ua, "rim tablet" ) != -1
             || ( index( $ua, "bb10" ) != -1
@@ -1627,7 +1669,7 @@ sub version {
     my ( $self, $check ) = @_;
     $self->_init_version() unless $self->{version_tests};
 
-    my $version = $self->{major} + $self->{minor};
+    my $version = "$self->{major}$self->{minor}";
     if ( defined $check ) {
         return $check
             == $version;    # FIXME unreliable to compare floats for equality
@@ -1668,7 +1710,7 @@ sub public_version {
     my ( $self,  $check ) = @_;
     my ( $major, $minor ) = $self->_public;
 
-    return $major + $minor;
+    return "$major$minor";
 }
 
 sub public_major {
@@ -1735,7 +1777,7 @@ sub _public {
             my ( $major, $minor ) = split /\./, $version;
             my $beta;
             $minor =~ s/(\D.*)// and $beta = $1;
-            $minor = 0 + ( '.' . $minor );
+            $minor = ( '.' . $minor );
             return ( $major, $minor, ( $beta ? 1 : undef ) );
         }
     }
@@ -1906,7 +1948,7 @@ sub _language_country {
 }
 
 sub _format_minor {
-    my $self = shift;
+    my $self  = shift;
     my $minor = shift;
 
     return 0 + ( '.' . ( $minor || 0 ) );
@@ -1974,7 +2016,7 @@ HTTP::BrowserDetect - Determine Web browser, version, and platform from an HTTP
 
 =head1 VERSION
 
-version 1.76
+version 1.77
 
 =head1 SYNOPSIS
 
@@ -2079,61 +2121,82 @@ looking for.
 
 =head2 public_version()
 
-Returns the browser version as a floating-point number.
+Returns the browser version (major and minor) as a string.
 
 =head2 public_major()
 
-Returns the integer portion of the browser version.
+Returns the major part of the version as a string. For example, for
+Chrome 36.0.1985.67, this returns "36".
+
+Returns undef if no version information can be detected.
 
 =head2 public_minor()
 
-Returns the decimal portion of the browser version as a B<floating-point
-number> less than 1. For example, if the version is 4.05, this method returns
-.05; if the version is 4.5, this method returns .5.
+Returns the minor part of the version as a string. This includes the
+decimal point; for example, for Chrome 36.0.1985.67, this returns
+".0".
 
-On occasion a version may have more than one decimal point, such as
-'Wget/1.4.5'. The minor version does not include the second decimal point, or
-any further digits or decimals.
+Returns undef if no version information can be detected.
+
+=head2 public_beta()
+
+Returns any part of the version after the major and minor version, as
+a string. For example, for Chrome 36.0.1985.67, this returns
+".1985.67". The beta part of the string can contain any type of
+alphanumeric characters.
+
+Returns undef if no version information can be detected. Returns an
+empty string if version information is detected but it contains only
+a major and minor version with nothing following.
 
 =head2 version($version)
 
 This is probably not what you want.  Please use either public_version() or
 engine_version() instead.
 
-Returns the version as a floating-point number. If passed a parameter, returns
-true if it is equal to the version specified by the user agent string.
+Returns the version as a string. If passed a parameter, returns true
+if it equals the browser major version.
+
+This function returns wrong values for some Safari versions, for
+compatibility with earlier code. public_version() returns correct
+version numbers for Safari.
 
 =head2 major($major)
 
-This is probably not what you want.  Please use either public_major() or
-engine_major() instead.
+This is probably not what you want. Please use either public_major()
+or engine_major() instead.
 
-Returns the integer portion of the browser version. If passed a parameter,
-returns true if it equals the browser major version.
+Returns the integer portion of the browser version as a string. If
+passed a parameter, returns true if it equals the browser major
+version.
+
+This function returns wrong values for some Safari versions, for
+compatibility with earlier code. public_version() returns correct
+version numbers for Safari.
 
 =head2 minor($minor)
 
-This is probably not what you want.  Please use either public_minor() or
-engine_minor() instead.
+This is probably not what you want. Please use either public_minor()
+or engine_minor() instead.
 
-Returns the decimal portion of the browser version as a B<floating-point
-number> less than 1. For example, if the version is 4.05, this method returns
-.05; if the version is 4.5, this method returns .5. B<This is a change in
-behavior from previous versions of this module, which returned a string>.
+Returns the decimal portion of the browser version as a string.
 
 If passed a parameter, returns true if equals the minor version.
 
-On occasion a version may have more than one decimal point, such as
-'Wget/1.4.5'. The minor version does not include the second decimal point, or
-any further digits or decimals.
+This function returns wrong values for some Safari versions, for
+compatibility with earlier code. public_version() returns correct
+version numbers for Safari.
 
 =head2 beta($beta)
 
-Returns any the beta version, consisting of any non-numeric characters after
-the version number. For instance, if the user agent string is 'Mozilla/4.0
-(compatible; MSIE 5.0b2; Windows NT)', returns 'b2'. If passed a parameter,
-returns true if equal to the beta version. If the beta starts with a dot, it
-is thrown away.
+This is probably not what you want. Please use public_beta() instead.
+
+Returns the beta version, consisting of any characters after the major
+and minor version number, as a string.
+
+This function returns wrong values for some Safari versions, for
+compatibility with earlier code. public_version() returns correct
+version numbers for Safari.
 
 =head1 Detecting Rendering Engine
 
@@ -2305,7 +2368,8 @@ Returns undef on failure.  Otherwise returns one of the following:
 
 Netscape, Firefox, Safari, Chrome, MSIE, WebTV, AOL Browser, Opera, Mosaic,
 Lynx, Links, ELinks, RealPlayer, IceWeasel, curl, puf, NetFront, Mobile Safari,
-BlackBerry, Obigo, Nintendo DSi, Nintendo 3DS, StarOffice, Lotus Notes, iCab.
+BlackBerry, Obigo, Nintendo DSi, Nintendo 3DS, StarOffice, Lotus Notes, iCab,
+BrowseX, Silk.
 
 =head2 gecko_version()
 
